@@ -98,7 +98,7 @@ typedef struct _QmiDevicePrivate QmiDevicePrivate;
 /**
  * QMI_DEVICE_SIGNAL_INDICATION:
  *
-* Symbol defining the #QmiDevice::indication signal.
+ * Symbol defining the #QmiDevice::indication signal.
  *
  * Since: 1.8
  */
@@ -137,7 +137,7 @@ GType qmi_device_get_type (void);
 /**
  * qmi_device_new:
  * @file: a #GFile.
- * @cancellable: optional #GCancellable object, #NULL to ignore.
+ * @cancellable: optional #GCancellable object, %NULL to ignore.
  * @callback: a #GAsyncReadyCallback to call when the initialization is finished.
  * @user_data: the data to pass to callback function.
  *
@@ -159,7 +159,7 @@ void qmi_device_new (GFile               *file,
  *
  * Finishes an operation started with qmi_device_new().
  *
- * Returns: A newly created #QmiDevice, or #NULL if @error is set.
+ * Returns: A newly created #QmiDevice, or %NULL if @error is set.
  *
  * Since: 1.0
  */
@@ -252,22 +252,24 @@ gboolean qmi_device_is_open (QmiDevice *self);
  * @QMI_DEVICE_OPEN_FLAGS_PROXY: Try to open the port through the 'qmi-proxy'. Since: 1.8.
  * @QMI_DEVICE_OPEN_FLAGS_MBIM: open an MBIM port with QMUX tunneling service. Since: 1.16.
  * @QMI_DEVICE_OPEN_FLAGS_AUTO: open a port either in QMI or MBIM mode, depending on device driver. Since: 1.18.
+ * @QMI_DEVICE_OPEN_FLAGS_EXPECT_INDICATIONS: Explicitly state that indications are wanted (implicit in QMI mode, optional when in MBIM mode).
  *
  * Flags to specify which actions to be performed when the device is open.
  *
  * Since: 1.0
  */
 typedef enum {
-    QMI_DEVICE_OPEN_FLAGS_NONE              = 0,
-    QMI_DEVICE_OPEN_FLAGS_VERSION_INFO      = 1 << 0,
-    QMI_DEVICE_OPEN_FLAGS_SYNC              = 1 << 1,
-    QMI_DEVICE_OPEN_FLAGS_NET_802_3         = 1 << 2,
-    QMI_DEVICE_OPEN_FLAGS_NET_RAW_IP        = 1 << 3,
-    QMI_DEVICE_OPEN_FLAGS_NET_QOS_HEADER    = 1 << 4,
-    QMI_DEVICE_OPEN_FLAGS_NET_NO_QOS_HEADER = 1 << 5,
-    QMI_DEVICE_OPEN_FLAGS_PROXY             = 1 << 6,
-    QMI_DEVICE_OPEN_FLAGS_MBIM              = 1 << 7,
-    QMI_DEVICE_OPEN_FLAGS_AUTO              = 1 << 8,
+    QMI_DEVICE_OPEN_FLAGS_NONE               = 0,
+    QMI_DEVICE_OPEN_FLAGS_VERSION_INFO       = 1 << 0,
+    QMI_DEVICE_OPEN_FLAGS_SYNC               = 1 << 1,
+    QMI_DEVICE_OPEN_FLAGS_NET_802_3          = 1 << 2,
+    QMI_DEVICE_OPEN_FLAGS_NET_RAW_IP         = 1 << 3,
+    QMI_DEVICE_OPEN_FLAGS_NET_QOS_HEADER     = 1 << 4,
+    QMI_DEVICE_OPEN_FLAGS_NET_NO_QOS_HEADER  = 1 << 5,
+    QMI_DEVICE_OPEN_FLAGS_PROXY              = 1 << 6,
+    QMI_DEVICE_OPEN_FLAGS_MBIM               = 1 << 7,
+    QMI_DEVICE_OPEN_FLAGS_AUTO               = 1 << 8,
+    QMI_DEVICE_OPEN_FLAGS_EXPECT_INDICATIONS = 1 << 9,
 } QmiDeviceOpenFlags;
 
 /**
@@ -281,7 +283,7 @@ typedef enum {
  * @self: a #QmiDevice.
  * @flags: mask of #QmiDeviceOpenFlags specifying how the device should be opened.
  * @timeout: maximum time, in seconds, to wait for the device to be opened.
- * @cancellable: optional #GCancellable object, #NULL to ignore.
+ * @cancellable: optional #GCancellable object, %NULL to ignore.
  * @callback: a #GAsyncReadyCallback to call when the operation is finished.
  * @user_data: the data to pass to callback function.
  *
@@ -315,28 +317,6 @@ gboolean qmi_device_open_finish (QmiDevice     *self,
                                  GAsyncResult  *res,
                                  GError       **error);
 
-/**
- * qmi_device_close:
- * @self: a #QmiDevice
- * @error: Return location for error or %NULL.
- *
- * Synchronously closes a #QmiDevice, preventing any further I/O.
- *
- * If this device was opened with @QMI_DEVICE_OPEN_FLAGS_MBIM, this
- * operation will not wait for the response of the underlying MBIM
- * close sequence.
- *
- * Closing a #QmiDevice multiple times will not return an error.
- *
- * Returns: %TRUE if successful, %FALSE if @error is set.
- *
- * Since: 1.0
- * Deprecated: 1.18: Use qmi_device_close_async() instead.
- */
-G_DEPRECATED
-gboolean qmi_device_close (QmiDevice  *self,
-                           GError    **error);
-
 /**
  * qmi_device_close_async:
  * @self: a #QmiDevice.
@@ -386,7 +366,7 @@ gboolean qmi_device_close_finish (QmiDevice     *self,
  * @service: a valid #QmiService.
  * @cid: a valid client ID, or #QMI_CID_NONE.
  * @timeout: maximum time to wait.
- * @cancellable: optional #GCancellable object, #NULL to ignore.
+ * @cancellable: optional #GCancellable object, %NULL to ignore.
  * @callback: a #GAsyncReadyCallback to call when the operation is finished.
  * @user_data: the data to pass to callback function.
  *
@@ -419,7 +399,7 @@ void qmi_device_allocate_client (QmiDevice           *self,
  *
  * Finishes an operation started with qmi_device_allocate_client().
  *
- * Returns: a newly allocated #QmiClient, or #NULL if @error is set.
+ * Returns: a newly allocated #QmiClient, or %NULL if @error is set.
  *
  * Since: 1.0
  */
@@ -453,7 +433,7 @@ typedef enum {
  * @client: the #QmiClient to release.
  * @flags: mask of #QmiDeviceReleaseClientFlags specifying how the client should be released.
  * @timeout: maximum time to wait.
- * @cancellable: optional #GCancellable object, #NULL to ignore.
+ * @cancellable: optional #GCancellable object, %NULL to ignore.
  * @callback: a #GAsyncReadyCallback to call when the operation is finished.
  * @user_data: the data to pass to callback function.
  *
@@ -487,7 +467,7 @@ void qmi_device_release_client (QmiDevice                   *self,
  * Note that even if the release operation returns an error, the client should
  * anyway be considered released, and shouldn't be used afterwards.
  *
- * Returns: %TRUE if successful, or #NULL if @error is set.
+ * Returns: %TRUE if successful, or %NULL if @error is set.
  *
  * Since: 1.0
  */
@@ -500,7 +480,7 @@ gboolean qmi_device_release_client_finish (QmiDevice     *self,
  * @self: a #QmiDevice.
  * @instance_id: the instance ID.
  * @timeout: maximum time to wait.
- * @cancellable: optional #GCancellable object, #NULL to ignore.
+ * @cancellable: optional #GCancellable object, %NULL to ignore.
  * @callback: a #GAsyncReadyCallback to call when the operation is finished.
  * @user_data: the data to pass to callback function.
  *
@@ -522,7 +502,7 @@ void qmi_device_set_instance_id (QmiDevice           *self,
  * qmi_device_set_instance_id_finish:
  * @self: a #QmiDevice.
  * @res: a #GAsyncResult.
- * @link_id: a placeholder for the output #guint16, or #NULL if not required.
+ * @link_id: a placeholder for the output #guint16, or %NULL if not required.
  * @error: Return location for error or %NULL.
  *
  * Finishes an operation started with qmi_device_set_instance_id().
@@ -547,6 +527,14 @@ gboolean qmi_device_set_instance_id_finish (QmiDevice     *self,
  *
  * Asynchronously sends a generic #QmiMessage to the device with no context.
  *
+ * If the operation is cancelled via @cancellable, a %QMI_PROTOCOL_ERROR_ABORTED
+ * error will be returned always. If the QMI method may be aborted, there is
+ * no guarantee that the operation is truly aborted before the error is returned
+ * so it may really happen that the operation really succeeded and the method
+ * would still return %QMI_PROTOCOL_ERROR_ABORTED. In order to use abortable
+ * methods and make sure the response is the correct one, use
+ * qmi_device_command_abortable().
+ *
  * When the operation is finished @callback will be called. You can then call
  * qmi_device_command_finish() to get the result of the operation.
  *
@@ -569,7 +557,7 @@ void qmi_device_command (QmiDevice           *self,
  *
  * Finishes an operation started with qmi_device_command().
  *
- * Returns: a #QmiMessage response, or #NULL if @error is set. The returned value should be freed with qmi_message_unref().
+ * Returns: a #QmiMessage response, or %NULL if @error is set. The returned value should be freed with qmi_message_unref().
  *
  * Since: 1.0
  * Deprecated: 1.18. Use qmi_device_command_full_finish() instead.
@@ -592,13 +580,19 @@ QmiMessage  *qmi_device_command_finish (QmiDevice     *self,
  * Asynchronously sends a #QmiMessage to the device.
  *
  * The message will be processed according to the specific @message_context
- * given.
+ * given. If no @context given, the behavior is the same as qmi_device_command().
+ *
+ * If the operation is cancelled via @cancellable, a %QMI_PROTOCOL_ERROR_ABORTED
+ * error will be returned always. If the QMI method may be aborted, there is
+ * no guarantee that the operation is truly aborted before the error is returned
+ * so it may really happen that the operation really succeeded and the method
+ * would still return %QMI_PROTOCOL_ERROR_ABORTED. In order to use abortable
+ * methods and make sure the response is the correct one, use
+ * qmi_device_command_abortable().
  *
  * When the operation is finished @callback will be called. You can then call
  * qmi_device_command_full_finish() to get the result of the operation.
  *
- * If no @context given, the behavior is the same as qmi_device_command().
- *
  * Since: 1.18
  */
 void qmi_device_command_full (QmiDevice           *self,
@@ -617,7 +611,7 @@ void qmi_device_command_full (QmiDevice           *self,
  *
  * Finishes an operation started with qmi_device_command_full().
  *
- * Returns: a #QmiMessage response, or #NULL if @error is set. The returned value should be freed with qmi_message_unref().
+ * Returns: a #QmiMessage response, or %NULL if @error is set. The returned value should be freed with qmi_message_unref().
  *
  * Since: 1.18
  */
@@ -625,6 +619,122 @@ QmiMessage *qmi_device_command_full_finish (QmiDevice     *self,
                                             GAsyncResult  *res,
                                             GError       **error);
 
+/**
+ * QmiDeviceCommandAbortableBuildRequestFn:
+ * @self: a #QmiDevice.
+ * @message: the #QmiMessage to abort.
+ * @user_data: the data provided when calling qmi_device_command_abortable().
+ * @error: Return location for error or %NULL.
+ *
+ * Callback to run when processing the command abortion. This callback
+ * should create a service-specific and client-specific abort request to
+ * be passed to the device.
+ *
+ * Returns: the abort request as a #QmiMessage or %NULL if @error is set.
+ *
+ * Since: 1.24
+ */
+typedef QmiMessage * (* QmiDeviceCommandAbortableBuildRequestFn) (QmiDevice   *self,
+                                                                  QmiMessage  *message,
+                                                                  gpointer     user_data,
+                                                                  GError     **error);
+
+/**
+ * QmiDeviceCommandAbortableParseResponseFn:
+ * @self: a #QmiDevice.
+ * @abort_response: the abort response #QmiMessage.
+ * @user_data: the data provided when calling qmi_device_command_abortable().
+ * @error: Return location for error or %NULL.
+ *
+ * Callback to run when processing the command abortion. This callback
+ * should parse the abort response provided by the device, and build an
+ * appropriate output.
+ *
+ * Returns: %TRUE if the abort succeeded, %FALSE if error is set.
+ *
+ * Since: 1.24
+ */
+typedef gboolean (* QmiDeviceCommandAbortableParseResponseFn) (QmiDevice   *self,
+                                                               QmiMessage  *abort_response,
+                                                               gpointer     user_data,
+                                                               GError     **error);
+
+/**
+ * qmi_device_command_abortable:
+ * @self: a #QmiDevice.
+ * @message: the message to send.
+ * @message_context: the context of the message.
+ * @timeout: maximum time, in seconds, to wait for the response.
+ * @abort_build_request_fn: callback to build an abort request.
+ * @abort_parse_response_fn: callback to parse an abort response.
+ * @abort_user_data: user data to pass to @build_request_fn and @parse_response_fn.
+ * @abort_user_data_free: a #GDestroyNotify to free @abort_user_data.
+ * @cancellable: a #GCancellable, or %NULL.
+ * @callback: a #GAsyncReadyCallback to call when the operation is finished.
+ * @user_data: the data to pass to callback function.
+ *
+ * Asynchronously sends a #QmiMessage to the device.
+ *
+ * The message will be processed according to the specific @message_context
+ * given.
+ *
+ * If the operation is cancelled via @cancellable, an abort message will
+ * be sent to the device in order to really abort the ongoing operation. The
+ * qmi_device_command_abortable() method will not finish until either a
+ * successful reply is received from the device or otherwise the command
+ * abortion is confirmed.
+ *
+ * If a successful command response arrives before the abort is processed by
+ * the device, the operation will succeed even if @cancellable has been set, so
+ * that upper layers can do whatever they need to do to properly tear down the
+ * operation.
+ *
+ * If an error command reponse arrives before the abort is processed by the
+ * device, the operation will fail with the error returned by the device.
+ *
+ * The %QMI_PROTOCOL_ERROR_ABORTED error will only be returned as a result
+ * of this operation if the command request was truly aborted by the device,
+ * and so, the user of the method should not assume that cancelling the
+ * @cancellable will always make the method return a %QMI_PROTOCOL_ERROR_ABORTED
+ * error.
+ *
+ * The @build_request_fn and @parse_response_fn callbacks are required in order
+ * to build the abort requests and parse the abort responses, because these
+ * are both service and client specific.
+ *
+ * When the operation is finished @callback will be called. You can then call
+ * qmi_device_command_abortable_finish() to get the result of the operation.
+ *
+ * Since: 1.24
+ */
+void qmi_device_command_abortable (QmiDevice                                *self,
+                                   QmiMessage                               *message,
+                                   QmiMessageContext                        *message_context,
+                                   guint                                     timeout,
+                                   QmiDeviceCommandAbortableBuildRequestFn   abort_build_request_fn,
+                                   QmiDeviceCommandAbortableParseResponseFn  abort_parse_response_fn,
+                                   gpointer                                  abort_user_data,
+                                   GDestroyNotify                            abort_user_data_free,
+                                   GCancellable                             *cancellable,
+                                   GAsyncReadyCallback                       callback,
+                                   gpointer                                  user_data);
+
+/**
+ * qmi_device_command_abortable_finish:
+ * @self: a #QmiDevice.
+ * @res: a #GAsyncResult.
+ * @error: Return location for error or %NULL.
+ *
+ * Finishes an operation started with qmi_device_command_abortable().
+ *
+ * Returns: a #QmiMessage response, or %NULL if @error is set. The returned value should be freed with qmi_message_unref().
+ *
+ * Since: 1.24
+ */
+QmiMessage *qmi_device_command_abortable_finish (QmiDevice     *self,
+                                                 GAsyncResult  *res,
+                                                 GError       **error);
+
 /**
  * QmiDeviceServiceVersionInfo:
  * @service: a #QmiService.
@@ -671,7 +781,7 @@ void qmi_device_get_service_version_info (QmiDevice           *self,
  *
  * Finishes an operation started with qmi_device_get_service_version_info().
  *
- * Returns: a #GArray of #QmiDeviceServiceVersionInfo elements, or #NULL if @error is set. The returned value should be freed with g_array_unref().
+ * Returns: a #GArray of #QmiDeviceServiceVersionInfo elements, or %NULL if @error is set. The returned value should be freed with g_array_unref().
  *
  * Since: 1.6
  */
@@ -727,7 +837,7 @@ QmiDeviceExpectedDataFormat qmi_device_get_expected_data_format (QmiDevice  *sel
  * Configures the data format currently expected by the kernel in the network
  * interface.
  *
- * Returns: %TRUE if successful, or #NULL if @error is set.
+ * Returns: %TRUE if successful, or %NULL if @error is set.
  *
  * Since: 1.14
  */
